@@ -1,15 +1,16 @@
-# $Id: ODBC.pm 11256 2008-05-12 14:20:24Z mjevans $
+# $Id: ODBC.pm 11768 2008-09-12 13:00:28Z mjevans $
 #
 # Copyright (c) 1994,1995,1996,1998  Tim Bunce
 # portions Copyright (c) 1997-2004  Jeff Urlwin
 # portions Copyright (c) 1997  Thomas K. Wenrich
+# portions Copyright (c) 2007-2008 Martin J. Evans
 #
 # You may distribute under the terms of either the GNU General Public
 # License or the Artistic License, as specified in the Perl README file.
 
 require 5.006;
 
-$DBD::ODBC::VERSION = '1.16';
+$DBD::ODBC::VERSION = '1.17';
 
 {
     package DBD::ODBC;
@@ -20,7 +21,7 @@ $DBD::ODBC::VERSION = '1.16';
 
     @ISA = qw(Exporter DynaLoader);
 
-    # my $Revision = substr(q$Id: ODBC.pm 11256 2008-05-12 14:20:24Z mjevans $, 13,2);
+    # my $Revision = substr(q$Id: ODBC.pm 11768 2008-09-12 13:00:28Z mjevans $, 13,2);
 
     require_version DBI 1.21;
 
@@ -33,7 +34,9 @@ $DBD::ODBC::VERSION = '1.16';
 
     sub parse_trace_flag {
         my ($class, $name) = @_;
-        return 0x01000000 if $name eq 'odbcdev';
+        #return 0x02000000 if name eq 'this_is_spare_use_me';
+        return 0x02000000 if $name eq 'odbcunicode';
+        return 0x04000000 if $name eq 'odbcconnection';
         return DBI::parse_trace_flag($class, $name);
     }
 
@@ -69,6 +72,7 @@ $DBD::ODBC::VERSION = '1.16';
 
 {   package DBD::ODBC::dr; # ====== DRIVER ======
     use strict;
+    use warnings;
 
     sub connect {
 	my $drh = shift;
@@ -96,6 +100,7 @@ $DBD::ODBC::VERSION = '1.16';
 
 {   package DBD::ODBC::db; # ====== DATABASE ======
     use strict;
+    use warnings;
 
     sub parse_trace_flag {
         my ($h, $name) = @_;
@@ -114,8 +119,10 @@ $DBD::ODBC::VERSION = '1.16';
                 odbc_cursortype => undef,
                 odbc_query_timeout => undef, # sth and dbh
                 odbc_has_unicode => undef,
+                odbc_out_connect_string => undef,
                 odbc_version => undef,
-                odbc_err_handler => undef
+                odbc_err_handler => undef,
+                odbc_putdata_start => undef # sth and dbh
                };
     }
 
@@ -436,6 +443,7 @@ $DBD::ODBC::VERSION = '1.16';
 
 {   package DBD::ODBC::st; # ====== STATEMENT ======
     use strict;
+    use warnings;
 
     *parse_trace_flag = \&DBD::ODBC::db::parse_trace_flag;
 
@@ -445,7 +453,8 @@ $DBD::ODBC::VERSION = '1.16';
                 odbc_default_bind_type => undef, # sth and dbh
                 odbc_force_rebind => undef, # sth & dbh
                 odbc_async_exec => undef, # sth and dbh
-                odbc_query_timeout => undef # sth and dbh
+                odbc_query_timeout => undef, # sth and dbh
+                odbc_putdata_start => undef # sth and dbh
                };
     }
 
@@ -523,6 +532,13 @@ get a list of checked in changes.
 
 =head2 Contributing
 
+There are six main ways you may help with the development and
+maintenance of this module:
+
+=over
+
+=item Submitting patches
+
 Please use Subversion (see above) to get the latest version of
 DBD::ODBC from the trunk and submit any patches against that.
 
@@ -536,13 +552,82 @@ Please, before submitting a patch:
 and send the resulting file to me and cc the dbi-users@perl.org
 mailing list (if you are not a member - why not!).
 
+=item Reporting installs
+
+Install CPAN::Reporter and report you installations. This is easy to
+do - see L</CPAN Testers Reporting>.
+
+=item Report bugs
+
+If you find what you believe is a bug then enter it into the
+L<http://rt.cpan.org> system. Where possible include code which reproduces the
+problem including any schema required and the versions of software you
+are using.
+
+If you are unsure whether you have found a bug report it anyway or
+post it to the dbi-users mailing list.
+
+=item pod comments and corrections
+
+If you find inaccuracies in the DBD::ODBC pod or have a comment which
+you think should be added then go to L<http://annocpan.org> and submit
+them there. I get an email for every comment added and will review
+each one and apply any changes to the documentation.
+
+=item Review DBD::ODBC
+
+Add your review of DBD::ODBC on L<http://cpanratings.perl.org>.
+
+If you are a member on ohloh then add your review or register your
+use of DBD::ODBC at L<http://www.ohloh.net/projects/perl_dbd_odbc>.
+
+=item submit test cases
+
+Most DBDs are built against a single client library for the database.
+
+Unlike other DBDs, DBD::ODBC works with many different ODBC drivers.
+Although they all should be written with regard to the ODBC
+specification drivers have bugs and in some places the specification is
+open to interpretation. As a result, when changes are applied to
+DBD::ODBC it is very easy to break something in one ODBC driver.
+
+What helps enormously to identify problems in the many combinations
+of DBD::ODBC and ODBC drivers is a large test suite. I would greatly
+appreciate any test cases and in particular any new test cases for
+databases other than MS SQL Server.
+
+=back
+
+=head2 DBI attribute handling
+
+If a DBI defined attribute is not mentioned here it behaves as per the
+DBI specification.
+
+=head3 ReadOnly (boolean)
+
+DBI documents the C<ReadOnly> attribute as being settleable and
+retrievable on connection and statement handles. In ODBC setting
+ReadOnly to true causes the connection attribute SQL_ATTR_ACCESS_MODE
+to be set to C<SQL_MODE_READ_ONLY> and setting it to false will set the
+access mode to C<SQL_MODE_READ_WRITE> (which is the default in ODBC).
+
+B<Note:> There is no equivalent of setting ReadOnly on a statement
+handle in ODBC.
+
+B<Note:> See ODBC documentation on C<SQL_ATTR_ACCESS_MODE> as setting it
+to C<SQL_MODE_READ_ONLY> does B<not> prevent your script from running
+updates or deletes; it is simply a hint to the driver/database that
+you won't being doing updates.
+
+This attribute requires DBI version 1.55 or better.
+
 =head2 Private attributes common to connection and statement handles
 
 =head3 odbc_ignore_named_placeholders
 
 Use this if you have special needs (such as Oracle triggers, etc)
 where :new or :name mean something special and are not just place
-holder names You I<must> then use ? for binding parameters.  Example:
+holder names. You I<must> then use ? for binding parameters.  Example:
 
  $dbh->{odbc_ignore_named_placeholders} = 1;
  $dbh->do("create trigger foo as if :new.x <> :old.x then ... etc");
@@ -554,12 +639,17 @@ binding and get confused.
 
 This value defaults to 0.
 
-Older versions of DBD::ODBC assumed that the binding type was 12
-(SQL_VARCHAR).  Newer versions default to 0, which means that
-DBD::ODBC will attempt to query the driver via SQLDescribeParam to
-determine the correct type.  If the driver doesn't support
-SQLDescribeParam, then DBD::ODBC falls back to using SQL_VARCHAR as
-the default, unless overridden by bind_param().
+Older versions of DBD::ODBC assumed that the parameter binding type
+was 12 (C<SQL_VARCHAR>).  Newer versions always attempt to call
+C<SQLDescribeParam> to find the parameter types but if
+C<SQLDescribeParam> is unavailable DBD::ODBC falls back to a default
+bind type. The internal default bind type is C<SQL_VARCHAR> (for
+non-unicode build) and C<SQL_WVARCHAR> (for a unicode build). If you
+set C<odbc_default_bind_type> to a value other than 0 you override the
+internal default.
+
+B<N.B> If you call the C<bind_param> method with a SQL type this
+overrides everything else above.
 
 =head3 odbc_force_rebind
 
@@ -578,16 +668,16 @@ about this.
 Allow asynchronous execution of queries.  This causes a spin-loop
 (with a small "sleep") until the SQL is complete.  This is useful,
 however, if you want the error handling and asynchronous messages (see
-the L</odbc_err_handler> and t/20SQLServer.t for an example of this.
+the L</odbc_err_handler> and F<t/20SQLServer.t> for an example of this.
 
 =head3 odbc_query_timeout
 
 This allows you to change the ODBC query timeout (the ODBC statement
-attribute SQL_ATTR_QUERY_TIMEOUT). ODBC defines the query time out as
+attribute C<SQL_ATTR_QUERY_TIMEOUT>). ODBC defines the query time out as
 the number of seconds to wait for a SQL statement to execute before
 returning to the application. A value of 0 (the default) means there
-is no time out. B<Do not confuse this with the ODBC attributes
-SQL_ATTR_LOGIN_TIMEOUT and SQL_ATTR_CONNECTION_TIMEOUT>. Add
+is no time out. Do not confuse this with the ODBC attributes
+C<SQL_ATTR_LOGIN_TIMEOUT> and C<SQL_ATTR_CONNECTION_TIMEOUT>. Add
 
   { odbc_query_timeout => 30 }
 
@@ -606,12 +696,28 @@ currently no way to retrieve the capped value back from the driver.
 
 Note that some drivers may not support this attribute.
 
-See t/20SqlServer.t for an example.
+See F<t/20SqlServer.t> for an example.
+
+=head3 odbc_putdata_start
+
+odbc_putdata_start defines the size at which DBD::ODBC uses C<SQLPutData>
+and C<SQLParamData> to send larger objects to the database instead of
+simply binding them as normal with C<SQLBindParameter>. It is mostly a
+placeholder for future changes allowing chunks of data to be sent to
+the database and there is little reason for anyone to change it
+currently.
+
+The default for odbc_putdata_start is 32768 because this value was
+hard-coded in DBD::ODBC until 1.16_1.
 
 =head2 Private connection attributes
 
 =head3 odbc_err_handler
 
+B<NOTE:> There should be no reason to use this now as there is a DBI
+attribute of a similar name. In future versions this attribute will
+be deleted.
+
 Allow errors to be handled by the application.  A call-back function
 supplied by the application to handle or ignore messages.
 
@@ -623,7 +729,7 @@ error is passed through the normal DBI error handling.
 
 This can also be used for procedures under MS SQL Server (Sybase too,
 probably) to obtain messages from system procedures such as DBCC.
-Check t/20SQLServer.t and t/10handler.t.
+Check F<t/20SQLServer.t> and F<t/10handler.t>.
 
   $dbh->{RaiseError} = 1;
   sub err_handler {
@@ -639,38 +745,42 @@ Check t/20SQLServer.t and t/10handler.t.
 
 =head3 odbc_SQL_ROWSET_SIZE
 
-Here is the information from the original patch, however, I've learned
-since from other sources that this could/has caused SQL Server to
-"lock up".  Please use at your own risk!
+Here is the information from the original patch, however, this
+could/has caused SQL Server to "lock up".  Please use at your own
+risk!
 
-SQL_ROWSET_SIZE attribute patch from Andrew Brown
-> There are only 2 additional lines allowing for the setting of
-> SQL_ROWSET_SIZE as db handle option.
->
-> The purpose to my madness is simple. SqlServer (7 anyway) by default
-> supports only one select statement at once (using std ODBC cursors).
-> According to the SqlServer documentation you can alter the default setting
-> of
-> three values to force the use of server cursors - in which case multiple
-> selects are possible.
->
-> The code change allows for:
-> $dbh->{SQL_ROWSET_SIZE} = 2;    # Any value > 1
->
-> For this very purpose.
->
-> The setting of SQL_ROWSET_SIZE only affects the extended fetch command as
-> far as I can work out and thus setting this option shouldn't affect
-> DBD::ODBC operations directly in any way.
->
-> Andrew
->
+C<SQL_ROWSET_SIZE> attribute patch from Andrew Brown
+
+  > There are only 2 additional lines allowing for the setting of
+  > SQL_ROWSET_SIZE as db handle option.
+  >
+  > The purpose to my madness is simple. SqlServer (7 anyway) by default
+  > supports only one select statement at once (using std ODBC cursors).
+  > According to the SqlServer documentation you can alter the default
+  > setting of three values to force the use of server cursors - in
+  > which case multiple selects are possible.
+  >
+  > The code change allows for:
+  > $dbh->{SQL_ROWSET_SIZE} = 2;    # Any value > 1
+  >
+  > For this very purpose.
+  >
+  > The setting of SQL_ROWSET_SIZE only affects the extended fetch
+  > command as far as I can work out and thus setting this option
+  > shouldn't affect DBD::ODBC operations directly in any way.
+  >
+  > Andrew
+  >
+
+In versions of SQL Server 2008 and later see "Multiple Active
+Statements (MAS)" in the DBD::ODBC::FAQ instead of using this attribute.
 
 =head3 odbc_exec_direct
 
-Force DBD::ODBC to use SQLExecDirect instead of SQLPrepare/SQLExecute.
+Force DBD::ODBC to use C<SQLExecDirect> instead of
+C<SQLPrepare>/C<SQLExecute>.
 
-There are drivers that only support SQLExecDirect and the DBD::ODBC
+There are drivers that only support C<SQLExecDirect> and the DBD::ODBC
 do() override does not allow returning result sets.  Therefore, the
 way to do this now is to set the attribute odbc_exec_direct.
 
@@ -690,6 +800,11 @@ and
 
     $dbh->{odbc_exec_direct} = 1;
 
+B<NOTE:> Even if you build DBD::ODBC with unicode support you can
+still not pass unicode strings to the prepare method if you also set
+odbc_exec_direct. This is a restriction in this attribute which is
+unavoidable.
+
 =head3 SQL_DRIVER_ODBC_VER
 
 This, while available via get_info() is captured here.  I may get rid
@@ -718,7 +833,7 @@ instead.  For example:
        $sth2->execute($row[0]);
     }
 
-See t/20SqlServer.t for an example.
+See F<t/20SqlServer.t> for an example.
 
 =head3 odbc_has_unicode
 
@@ -727,10 +842,16 @@ C macro WITH_UNICODE or not. A value of 1 indicates DBD::ODBC was built
 with WITH_UNICODE else the value returned is 0.
 
 Building WITH_UNICODE affects columns and parameters which are
-SQL_C_WCHAR, SQL_WCHAR, SQL_WVARCHAR, and SQL_WLONGVARCHAR.
+SQL_C_WCHAR, SQL_WCHAR, SQL_WVARCHAR, and SQL_WLONGVARCHAR, SQL,
+the connect method and a lot more. See L</Unicode>.
 
 When odbc_has_unicode is 1, DBD::ODBC will:
 
+=head3 odbc_out_connect_string
+
+After calling the connect method this will be the ODBC driver's
+out connection string - see documentation on SQLDriverConnect.
+
 =over
 
 =item bind columns the database declares as wide characters as SQL_Wxxx
@@ -938,13 +1059,38 @@ Handled as of version 0.28
 head3 ColAttributes
 
 B<This private function is now superceded by DBI's statement attributes
-NAME, TYPE, PRECISION, SCLARE, NULLABLE etc).>
+NAME, TYPE, PRECISION, SCALE, NULLABLE etc).>
 
 See the ODBC specification for the SQLColAttributes API.
 You call SQLColAttributes like this:
 
   $dbh->func($column, $ftype, "ColAttributes");
 
+  SQL_COLUMN_COUNT = 0
+  SQL_COLUMN_NAME = 1
+  SQL_COLUMN_TYPE = 2
+  SQL_COLUMN_LENGTH = 3
+  SQL_COLUMN_PRECISION = 4
+  SQL_COLUMN_SCALE = 5
+  SQL_COLUMN_DISPLAY_SIZE = 6
+  SQL_COLUMN_NULLABLE = 7
+  SQL_COLUMN_UNSIGNED = 8
+  SQL_COLUMN_MONEY = 9
+  SQL_COLUMN_UPDATABLE = 10
+  SQL_COLUMN_AUTO_INCREMENT = 11
+  SQL_COLUMN_CASE_SENSITIVE = 12
+  SQL_COLUMN_SEARCHABLE = 13
+  SQL_COLUMN_TYPE_NAME = 14
+  SQL_COLUMN_TABLE_NAME = 15
+  SQL_COLUMN_OWNER_NAME = 16
+  SQL_COLUMN_QUALIFIER_NAME = 17
+  SQL_COLUMN_LABEL = 18
+
+B<Note:>Oracle's ODBC driver for linux in instant client 11r1 often
+returns strange values for column name e.g., '20291'. It is wiser to
+use DBI's NAME and NAME_xx attributes for portability.
+
+
 head3 DescribeCol
 
 B<This private function is now superceded by DBI's statement attributes
@@ -966,27 +1112,36 @@ of DBI 1.604, the only trace flag defined which is relevant to
 DBD::ODBC is 'SQL' which DBD::ODBC supports by outputting the SQL
 strings (after modification) passed to the prepare and do methods.
 
-Currently DBD::ODBC only supports one private trace flag.  The
-'odbcdev' trace flag is used to output development tracing so it
-should not be relevant in normal use.
+Currently DBD::ODBC supports two private trace flags. The
+'odbcunicode' flag traces some unicode operations and the
+odbcconnection traces the connect process.
 
 To enable tracing of particular flags you use:
 
-  $h->trace($h->parse_trace_flags('SQL|odbcdev'));
-  $h->trace($h->parse_trace_flags('1|odbcdev'));
+  $h->trace($h->parse_trace_flags('SQL|odbcconnection'));
+  $h->trace($h->parse_trace_flags('1|odbcunicode'));
 
-In the first case 'SQL' and 'odbcdev' tracing is enabled on $h. In the
-second case trace level 1 is set and 'odbcdev' tracing is enabled.
+In the first case 'SQL' and 'odbcconnection' tracing is enabled on
+$h. In the second case trace level 1 is set and 'odbcunicode' tracing
+is enabled.
 
 If you want to enable a DBD::ODBC private trace flag before connecting
 you need to do something like:
 
   use DBD::ODBC;
-  DBI->trace(DBD::ODBC->parse_trace_flag('odbcdev');
+  DBI->trace(DBD::ODBC->parse_trace_flag('odbcconnection'));
+
+or
+
+  use DBD::ODBC;
+  DBI->trace(DBD::ODBC->parse_trace_flags('odbcconnection|odbcunicode'));
 
 DBD::ODBC outputs tracing at levels 3 and above (as levels 1 and 2 are
 reserved for DBI).
 
+For comprehensive tracing of DBI method calls without all the DBI
+internals see L<DBIx::Log4perl>.
+
 =head2 Deviations from the DBI specification
 
 =head3 Mixed placeholder types
@@ -1002,7 +1157,7 @@ an error like:
 
 =head3 Using the same placeholder more than once
 
-DBD::ODBC does not support (currently) the use of one placeholder
+DBD::ODBC does not support (currently) the use of one named placeholder
 more than once in the a single SQL string. i.e.,
 
   insert into foo values (:bar, :p1, :p2, :bar);
@@ -1034,12 +1189,294 @@ In discussion on the dbi-dev list is was suggested that the ':' could
 be made optional and there were no basic objections but it has not
 made it's way into the pod yet.
 
+=head2 Unicode
+
+The ODBC specification supports wide character versions (a postfix of
+'W') of some of the normal ODBC APIs e.g., SQLDriverConnectW is a wide
+character version of SQLDriverConnect.
+
+In ODBC on Windows the wide characters are defined as SQLWCHARs (2
+bytes) and are UCS-2. On non-Windows, the main driver managers I know
+of have implemented the wide character APIs differently:
+
+=over
+
+=item unixODBC
+
+unixODBC mimics the Windows ODBC API precisely meaning the wide
+character versions expect and return 2-byte characters in
+UCS-2.
+
+unixODBC will happily recognise ODBC drivers which only have the ANSI
+versions of the ODBC API and those that have the wide versions
+too.
+
+unixODBC will allow an ANSI application to work with a unicode
+ODBC driver and vice versa (although in the latter case you obviously
+cannot actually use unicode).
+
+unixODBC does not prevent you sending UTF-8 in the ANSI versions of
+the ODBC APIs but whether that is understood by your ODBC driver is
+another matter.
+
+unixODBC differs in only one way from the Microsoft ODBC driver in
+terms of unicode support in that it avoids unnecessary translations
+between single byte and double byte characters when an ANSI
+application is using a unicode-aware ODBC driver by requiring unicode
+applications to signal their intent by calling SQLDriverConnectW
+first. On Windows, the ODBC driver manager always uses the wide
+versions of the ODBC API in ODBC drivers which provide the wide
+versions regardless of what the application really needs and this
+results in a lot of unnecessary character translations when you have
+an ANSI application and a unicode ODBC driver.
+
+=item iODBC
+
+The wide character versions expect and return wchar_t types.
+
+=back
+
+DBD::ODBC has gone with unixODBC so you cannot use iODBC with a
+unicode build of DBD::ODBC. However, some ODBC drivers support UTF-8
+(although how they do this with SQLGetData reliably I don't know)
+and so you should be able to use those with DBD::ODBC not built for
+unicode.
+
+=head3 Enabling and Disabling Unicode support
+
+On Windows Unicode support as ss enabled by default and to disable it
+you will need to specify C<-nou> to F<Makefile.PL> to get back to the
+original behavior of DBD::ODBC before any Unicode support was added.
+
+e.g.,
+
+  perl Makfile.PL -nou
+
+On non-Windows platforms Unicode support is disabled by default. To
+enable it specify C<-u> to F<Makefile.PL> when you configure DBD::ODBC.
+
+e.g.,
+
+  perl Makefile.PL -u
+
+=head3 Unicode - What is supported?
+
+As of version 1.17 DBD::ODBC has the following unicode support:
+
+=over
+
+=item SQL (introduced in 1.16_2)
+
+Unicode strings in calls to the C<prepare> and C<do> methods are
+supported so long as the C<odbc_execdirect> attribute is not used.
+
+=item unicode connection strings (introduced in 1.16_2)
+
+Unicode connection strings are supported but you will need a DBI
+post 1.607 for that.
+
+=item column names
+
+Unicode column names are returned.
+
+=item bound columns (introduced in 1.15)
+
+If the DBMS reports the column as being a wide character (SQL_Wxxx) it
+will be bound as a wide character and any returned data will be
+converted from UTF16 to UTF8 and the UTF8 flag will then be set on the
+data.
+
+=item bound parameters
+
+If the perl scalars you bind to parameters are marked UTF8 and the
+DBMS reports the type as being a wide type or you bind the parameter
+as a wide type they will be converted to wide characters and bound as
+such.
+
+=back
+
+Since version 1.16_4, the default parameter bind type is SQL_WVARCHAR
+for unicode builds of DBD::ODBC. This only affects ODBC drivers which
+do not support SQLDescribeParam and only then if you do not
+specifically set a sql type on the bind_param method call.
+
+The above Unicode support has been tested with the SQL Server, Oracle
+9.2+ and Postgres drivers on Windows and various Easysoft ODBC drivers
+on UNIX.
+
+=head3 Unicode - What is not supported?
+
+You cannot use unicode parameter names e.g.,
+
+  select * from table where column = :unicode_param_name
+
+You cannot use unicode strings in calls to prepare if you set the
+odbc_execdirect attribute.
+
+You cannot use the iODBC driver manager with DBD::ODBC built for
+unicode.
+
+=head3 Unicode - Caveats
+
+For Unicode support on any platform in Perl you will need at least
+Perl 5.8.1 - sorry but this is the way it is with Perl.
+
+The Unicode support in DBD::ODBC expects a WCHAR to be 2 bytes (as it
+is on Windows and as the ODBC specification suggests it is). Until
+ODBC specifies any other Unicode support it is not envisioned this
+will change.  On UNIX there are a few different ODBC driver
+managers. I have only tested the unixODBC driver manager
+(http://www.unixodbc.org) with Unicode support and it was built with
+defaults which set WCHAR as 2 bytes.
+
+I believe that the iODBC driver manager expects wide characters to be
+wchar_t types (which are usually 4) and hence DBD::ODBC will not work
+iODBC when built for unicode.
+
+The ODBC Driver must expect Unicode data specified in SQLBindParameter
+and SQLBindCol to be UTF16 in local endianess. Similarly, in calls to
+SQLPrepareW, SQLDescribeColW and SQLDriverConnectW.
+
+You should be aware that once Unicode support is enabled it affects a
+number of DBI methods (some of which you might not expect). For
+instance, when listing tables, columns etc some drivers
+(e.g. Microsoft SQL Server) will report the column types as wide types
+even if the strings actually fit in 7-bit ASCII. As a result, there is
+an overhead for retrieving this column data as 2 bytes per character
+will be transmitted (compared with 1 when Unicode support is not
+enabled) and these strings will be converted into UTF8 but will end up
+fitting (in most cases) into 7bit ASCII so a lot of conversion work
+has been performed for nothing. If you don't have Unicode table and
+column names or Unicode column data in your tables you are best
+disabling Unicode support.
+
+I am at present unsure if ChopBlanks processing on Unicode strings is
+working correctly on UNIX. If nothing else the construct L' ' in
+dbdimp.c might not work with all UNIX compilers. Reports of issues and
+patches welcome.
+
+=head3 Unicode implementation in DBD::ODBC
+
+DBD::ODBC uses the wide character versions of the ODBC API and the
+SQL_WCHAR ODBC type to support unicode in Perl.
+
+Wide characters returned from the ODBC driver will be converted to
+UTF-8 and the perl scalars will have the utf8 flag set (by using
+sv_utf8_decode).
+
+perl scalars which are UTF-8 and are sent through the ODBC API will be
+converted to UTF-16 and passed to the ODBC wide APIs or signalled as
+SQL_WCHARs (e.g., in the case of bound columns).
+
+When built for unicode, DBD::ODBC will always call SQLDriverConnectW
+(and not SQLDriverConnect) even if a) your connection string is not
+unicode b) you have not got a DBI later than 1.607, because unixODBC
+requires SQLDriverConnectW to be called if you want to call other
+unicode ODBC APIs later. As a result, if you build for unicode and
+pass ASCII strings to the connect method they will be converted to
+UTF-16 and passed to SQLDriverConnectW. This should make no real
+difference to perl not using unicode connection strings.
+
+You will need a DBI later than 1.607 to support unicode connection
+strings because until post 1.607 there was no way for DBI to pass
+unicode strings to the DBD.
+
+=head3 Unicode and Oracle
+
+You have to set the environment variables C<NLS_NCHAR=AL32UTF8> and
+C<NLS_LANG=AMERICAN_AMERICA.AL32UTF8> (or any other language setting
+ending with C<.AL32UTF8>) before loading DBD::ODBC to make Oracle
+return Unicode data. (See also "Oracle and Unicode" in the POD of
+DBD::Oracle.)
+
+On Windows, using the Oracle ODBC Driver you have to enable the B<Force
+SQL_WCHAR support> Workaround in the data source configuration to make
+Oracle return Unicode to a non-Unicode application. Alternatively, you
+can include C<FWC=T> in your connect string.
+
+Unless you need to use ODBC, if you want Unicode support with Oracle
+you are better off using L<DBD::Oracle>.
+
+=head3 Unicode and PostgreSQL
+
+Some tests from the original DBD::ODBC 1.13 fail with PostgreSQL
+8.0.3, so you may not want to use DBD::ODBC to connect to PostgreSQL
+8.0.3.
+
+Unicode tests fail because PostgreSQL seems not to give any hints
+about Unicode, so all data is treated as non-Unicode.
+
+Unless you need to use ODBC, if you want Unicode support with Postgres
+you are better off with L<DBD::Pg> as it has a specific attribute named
+C<pg_enable_utf8> to enable Unicode support.
+
+=head3 Unicode and Easysoft ODBC Drivers
+
+We have tested the Easysoft SQL Server, Oracle and ODBC Bridge drivers
+with DBD::ODBC built for Unicode. All work as described without
+modification except for the Oracle driver you will need to set you
+NLS_LANG as mentioned above.
+
+=head3 Unicode and other ODBC drivers
+
+If you have a unicode-enabled ODBC driver and it works with DBD::ODBC
+let me know and I will include it here.
+
+=head2 ODBC Support in ODBC Drivers
+
+=head3 Drivers without SQLDescribeParam
+
+Some drivers do not support the C<SQLDescribeParam> ODBC API (e.g.,
+Microsoft Access).
+
+DBD::ODBC uses the C<SQLDescribeParam> API when parameters are bound
+to your SQL to find the types of the parameters. If the ODBC driver
+does not support C<SQLDescribeParam>, DBD::ODBC assumes the parameters
+are C<SQL_VARCHAR> or C<SQL_WVARCHAR> types (depending on whether
+DBD::ODBC is built for unicode or not). In any case, if you bind a
+parameter and specify a SQL type this overrides any type DBD::ODBC
+would choose.
+
+For ODBC drivers which do not support C<SQLDescribeParam> the default
+behavior in DBD::ODBC may not be what you want. To change the default
+parameter bind type set L</odbc_default_bind_type>. If, after that you
+have some SQL where you need to vary the parameter types used add the
+SQL type to the end of the C<bind_param> method.
+
+  use DBI qw(:sql_types);
+  $h = DBI->connect;
+  # set the default bound parameter type
+  $h->{odbc_default_bind_type} = SQL_VARCHAR;
+  # bind a parameter with a specific type
+  $s = $h->prepare(q/insert into mytable values(?)/);
+  $s->bind_param(1, "\x{263a}", SQL_WVARCHAR);
+
+=head2 CPAN Testers Reporting
+
+Please, please, please (is that enough), consider installing
+CPAN::Reporter so that when you install perl modules a report of the
+installation success or failure can be sent to cpan testers. In this
+way module authors 1) get feedback on the fact that a module is being
+installed 2) get to know if there are any installation problems. Also
+other people like you may look at the test reports to see how
+successful they are before choosing the version of a module to
+install.
+
+CPAN::Reporter is easy to install and configure like this:
+
+  perl -MCPAN -e shell
+  cpan> install CPAN::Reporter
+  cpan> reload cpan
+  cpan> o conf init test_report
+
+Simply answer the questions to configure CPAN::Reporter.
+
+You can find the CPAN testers wiki at L<http://wiki.cpantesters.org/>
+and the installation guide for CPAN::Reporter at
+L<http://wiki.cpantesters.org/wiki/CPANInstall>.
+
 =head2 Others/todo?
 
-Level 1
-
-    SQLTables (use tables()) call
-
 Level 2
 
     SQLColumnPrivileges
@@ -1049,91 +1486,6 @@ Level 2
     SQLDrivers
     SQLNativeSql
 
-=head2 Connect without DSN
-
-The ability to connect without a full DSN is introduced in version
-0.21.
-
-Example (using MS Access):
-	my $DSN = 'driver=Microsoft Access Driver (*.mdb);dbq=\\\\cheese\\g$\\perltest.mdb';
-	my $dbh = DBI->connect("dbi:ODBC:$DSN", '','')
-		or die "$DBI::errstr\n";
-
-=head2 Using DBD::ODBC with web servers under Win32.
-
-=over 4
-
-=item General Commentary re web database access
-
-This should be a DBI faq, actually, but this has somewhat of an
-Win32/ODBC twist to it.
-
-Typically, the Web server is installed as an NT service or a Windows
-95/98 service.  This typically means that the web server itself does
-not have the same environment and permissions the web developer does.
-This situation, of course, can and does apply to Unix web servers.
-Under Win32, however, the problems are usually slightly different.
-
-=item Defining your DSN -- which type should I use?
-
-Under Win32 take care to define your DSN as a system DSN, not as a user
-DSN.  The system DSN is a "global" one, while the user is local to a
-user.  Typically, as stated above, the web server is "logged in" as a
-different user than the web developer.  This helps cause the situation
-where someone asks why a script succeeds from the command line, but
-fails when called from the web server.
-
-=item Defining your DSN -- careful selection of the file itself is important!
-
-For file based drivers, rather than client server drivers, the file
-path is VERY important.  There are a few things to keep in mind.  This
-applies to, for example, MS Access databases.
-
-1) If the file is on an NTFS partition, check to make sure that the Web
-B<service> user has permissions to access that file.
-
-2) If the file is on a remote computer, check to make sure the Web
-B<service> user has permissions to access the file.
-
-3) If the file is on a remote computer, try using a UNC path the file,
-rather than a X:\ notation.  This can be VERY important as services
-don't quite get the same access permissions to the mapped drive letters
-B<and>, more importantly, the drive letters themselves are GLOBAL to
-the machine.  That means that if the service tries to access Z:, the Z:
-it gets can depend upon the user who is logged into the machine at the
-time.  (I've tested this while I was developing a service -- it's ugly
-and worth avoiding at all costs).
-
-Unfortunately, the Access ODBC driver that I have does not allow one to
-specify the UNC path, only the X:\ notation.  There is at least one way
-around that.  The simplest is probably to use Regedit and go to
-(assuming it's a system DSN, of course)
-HKEY_LOCAL_USERS\SOFTWARE\ODBC\"YOUR DSN" You will see a few settings
-which are typically driver specific.  The important value to change for
-the Access driver, for example, is the DBQ value.  That's actually the
-file name of the Access database.
-
-=head2 Connect without DSN
-
-The ability to connect without a full DSN is introduced in version 0.21.
-
-Example (using MS Access):
-	my $DSN = 'driver=Microsoft Access Driver
-(*.mdb);dbq=\\\\cheese\\g$\\perltest.mdb';
-	my $dbh = DBI->connect("dbi:ODBC:$DSN", '','')
-		or die "$DBI::errstr\n";
-
-The above sample uses Microsoft's UNC naming convention to point to the MSAccess
-file (\\\\cheese\\g$\\perltest.mdb).  The dbq parameter tells the access driver
-which file to use for the database.
-
-Example (using MSSQL Server):
-      my $DSN = 'driver={SQL Server};Server=server_name;
-      database=database_name;uid=user;pwd=password;';
-      my $dbh  = DBI->connect("dbi:ODBC:$DSN") or die "$DBI::errstr\n";
-
-=back
-
 =head2 Random Links
 
 These are in need of sorting and annotating. Some are relevant only
@@ -1143,6 +1495,8 @@ You can find DBD::ODBC on ohloh now at:
 
 L<http://www.ohloh.net/projects/perl_dbd_odbc>
 
+If you use ohloh and DBD::ODBC please say you use it and rate it.
+
 There is a good search engine for the various Perl DBI lists at the
 following URLS:
 
@@ -1171,7 +1525,7 @@ L<http://www.openlinksw.com>
 
 L<http://www.datadirect.com>
 
-L<http://www.atinet.com/support/openrda_samples.asp>
+L<http://www.atinet.com>
 
 Some useful tutorials:
 
@@ -1190,7 +1544,7 @@ L<http://www.easysoft.com/developer/languages/perl/dbd_odbc_tutorial_part_1.html
 
 Perl DBI/DBD::ODBC Tutorial Part 2 - Introduction to retrieving data from your database:
 
-L<http://www.easysoft.com/developer/languages/perl/index.html>
+L<http://www.easysoft.com/developer/languages/perl/dbd_odbc_tutorial_part_2.html>
 
 Perl DBI/DBD::ODBC Tutorial Part 3 - Connecting Perl on UNIX or Linux to Microsoft SQL Server:
 
@@ -1202,7 +1556,36 @@ L<http://www.easysoft.com/developer/languages/perl/tutorial_data_web.html>
 
 =head2 Frequently Asked Questions
 
-Frequently asked questions are now in DBD::ODBC::FAQ. Run
+Frequently asked questions are now in L<DBD::ODBC::FAQ>. Run
 C<perldoc DBD::ODBC::FAQ> to view them.
 
+=head1 BUGS
+
+None known other than the deviations from the DBI specification mentioned
+above in L</Deviations from the DBI specification>.
+
+Please report any to me via the CPAN RT system. See
+L<http://rt.cpan.org/> for more details.
+
+=head1 AUTHOR
+
+Tim Bunce
+
+Jeff Urlwin
+
+Thomas K. Wenrich
+
+Martin J. Evans
+
+=head1 LICENSE
+
+This program is free software; you can redistribute it and/or modify
+it under the same terms as Perl itself.
+
+See L<http://www.perl.com/perl/misc/Artistic.html>
+
+=head1 SEE ALSO
+
+L<DBI>
+
 =cut
